/*
 *   SPDX-FileCopyrightText: 2019 Jonathan Rubenstein <jrubcop@gmail.com>
 *
 *   SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef BINARY_DATA_P_H
#define BINARY_DATA_P_H

#include "unitcategory_p.h"

namespace KUnitConversion
{

class BinaryData : public CustomCategory
{
public:
    BinaryData();
};

} // KUnitConversion namespace

#endif
